# -*- coding: utf-8 -*-
from ast import NodeVisitor

__version__ = "0.1.4"

SHIRE_MODULES = ("luz", "luzong", "corelib")

MESSAGE = "SH00 shire old style import found"


class ShireImportVisitor(NodeVisitor):
    def __init__(self):
        super(ShireImportVisitor, self).__init__()
        self.imports = []

    def visit_ImportFrom(self, node):
        n_modules = node.module.split(".") if node.module else []
        if len(set(n_modules).intersection(SHIRE_MODULES)) > 0:
            self.imports.append(node)

    def visit_Import(self, node):
        for n_name in node.names:
            if n_name.name in SHIRE_MODULES:
                self.imports.append(node)


class ShireChecker:
    name = "flaked-shire"
    version = __version__

    def __init__(self, tree, filename):
        self.tree = tree
        self.filename = filename

    def run(self):
        visitor = ShireImportVisitor()
        visitor.visit(self.tree)
        if not visitor.imports:
            return

        for import_node in visitor.imports:
            yield (
                import_node.lineno,
                0,
                MESSAGE,
                type(self),
            )



# -*- coding: utf-8 -*-
import argparse
import os
import time

from flake8.checker import LOG, FileChecker, Manager
from flake8.main import options
from flake8.main.application import Application as Flake8Application
from flake8.options import manager
from flake8.processor import FileProcessor
from pygments import lex
from pygments.lexers import MakoLexer
from pygments.token import string_to_tokentype

__version__ = "0.2.1"


def is_import_start(token, value):
    return token == string_to_tokentype("Token.Comment.Preproc") and value == "<%!"


def is_import_end(token, value):
    return token == string_to_tokentype("Token.Comment.Preproc") and value == "%>"


def parse_mako(source):
    imports_section = []
    tokens = lex(source, MakoLexer())

    is_import = False
    for token, value in tokens:
        if is_import_start(token, value):
            is_import = True
            continue
        if is_import_end(token, value):
            break

        if is_import:
            imports_section.append(value)

    # TODO: maybe better
    imports_section = "".join(imports_section).strip().split("\n")
    return [imp.strip() + "\n" for imp in imports_section]


def parse_file(filename, source):
    mako_ext = [".mako", ".html"]
    _, ext = os.path.splitext(filename)
    return parse_mako(source) if ext in mako_ext else None


# TODO: multiple files not work
class DManager(Manager):
    def make_checkers(self, paths=None):
        super(DManager, self).make_checkers(paths)

        new_checkers = []
        for checker in self.checkers:
            # parse file: if mako file then return python code lines
            source = "".join(checker.processor.read_lines())
            mako_lines = parse_file(checker.filename, source)
            _checker = DFileChecker(
                checker.filename,
                checker.checks,
                self.options,
                mako_lines=mako_lines,
            )
            new_checkers.append(_checker)

        self.checkers = new_checkers
        self._all_checkers = new_checkers
        LOG.info("Checking %s files", len(self.checkers))


class DFileChecker(FileChecker):
    def __init__(self, filename, checks, options, mako_lines=None):
        self.mako_lines = mako_lines
        super(DFileChecker, self).__init__(filename, checks, options)

    def _make_processor(self):
        try:
            return FileProcessor(self.filename, self.options, lines=self.mako_lines)
        except IOError as e:
            message = "{0}: {1}".format(type(e).__name__, e)
            self.report("E902", 0, 0, message)
            return


class Application(Flake8Application):
    def __init__(self, program="flaked", version=__version__):
        self.start_time = time.time()
        self.end_time = None
        self.program = program
        self.version = version
        self.prelim_arg_parser = argparse.ArgumentParser(add_help=False)
        options.register_preliminary_options(self.prelim_arg_parser)

        self.option_manager = manager.OptionManager(
            prog=program,
            version=version,
            parents=[self.prelim_arg_parser],
        )
        options.register_default_options(self.option_manager)

        self.check_plugins = None
        self.formatting_plugins = None
        self.formatter = None
        self.guide = None
        self.file_checker_manager = None
        self.options = None
        self.args = None
        self.result_count = 0
        self.total_result_count = 0
        self.catastrophic_failure = False
        self.running_against_diff = False
        self.parsed_diff = {}

    def make_file_checker_manager(self):
        self.file_checker_manager = DManager(
            style_guide=self.guide,
            arguments=self.args,
            checker_plugins=self.check_plugins,
        )



# -*- coding: utf-8 -*-

# Note: To use the 'upload' functionality of this file, you must:
#   $ pipenv install twine --dev

import io
import os
import sys
from shutil import rmtree
from flaked import __version__

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = "flaked"
DESCRIPTION = "flaked add mako file's py checker and more base on flake8"
URL = "https://github.com/ischaojie/flaked"
EMAIL = "zhuzhezhe95@gmail.com"
AUTHOR = "ischaojie"
REQUIRES_PYTHON = ">=2.7.0"
VERSION = "0.1.0"

REQUIRED = [
    "flake8 == 3.9.2",
    "pygments == 2.5.2",
]

EXTRAS = {}

here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, "README.md"), encoding="utf-8") as f:
        long_description = "\n" + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION


class UploadCommand(Command):
    """Support setup.py upload."""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(here, "dist"))
        except OSError:
            pass

        self.status("Building Source and Wheel (universal) distribution…")
        os.system("{0} setup.py sdist bdist_wheel --universal".format(sys.executable))

        self.status("Uploading the package to PyPI via Twine…")
        os.system("twine upload dist/*")

        self.status("Pushing git tags…")
        os.system("git tag v{0}".format(__version__))
        os.system("git push --tags")

        sys.exit()


# Where the magic happens:
setup(
    name=NAME,
    version=__version__,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    entry_points={
        "console_scripts": ["flaked=flaked.cli:main"],
        "flake8.extension": ["SH = flaked.plugins.flaked_shire:ShireChecker"],
    },
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    # $ setup.py publish support.
    cmdclass={
        "upload": UploadCommand,
    },
)
